@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -304,6 +304,9 @@ typedef struct
 									 * that health_check_period required to be
 									 * greater than 0 to enable the
 									 * functionality. */
+
+	bool		prefer_lower_delay_standby;
+
 	LogStandbyDelayModes log_standby_delay; /* how to log standby lag */
 	bool		connection_cache;	/* cache connection pool? */
 	int			health_check_timeout;	/* health check timeout */
@@ -338,6 +341,8 @@ typedef struct
 											 * set to false, pgpool will
 											 * report an error and disconnect
 											 * the session. */
+	bool		failover_on_backend_shutdown; /* If true, trigger fail over
+												 when backend is going down */
 	bool		detach_false_primary;	/* If true, detach false primary */
 	char	   *recovery_user;	/* PostgreSQL user name for online recovery */
 	char	   *recovery_password;	/* PostgreSQL user password for online
@@ -363,8 +368,6 @@ typedef struct
 	bool		log_statement;	/* logs all SQL statements */
 	bool		log_per_node_statement; /* logs per node detailed SQL
 										 * statements */
-	bool		notice_per_node_statement; /* logs notice message for per node detailed SQL
-										 * statements */
 	bool		log_client_messages;	/* If true, logs any client messages */
 	char	   *lobj_lock_table;	/* table name to lock for rewriting
 									 * lo_creat */
@@ -470,7 +473,7 @@ typedef struct
 	char	   *database_redirect_preference_list;	/* raw string in
 													 * pgpool.conf */
 	RegArray   *redirect_dbnames;	/* Precompiled regex patterns for db
-									 * prefrence list */
+									 * preference list */
 	Left_right_tokens *db_redirect_tokens;	/* db redirect for dbname and node
 											 * string */
 
@@ -481,7 +484,7 @@ typedef struct
 	char	   *app_name_redirect_preference_list;	/* raw string in
 													 * pgpool.conf */
 	RegArray   *redirect_app_names; /* Precompiled regex patterns for app name
-									 * prefrence list */
+									 * preference list */
 	Left_right_tokens *app_name_redirect_tokens;	/* app name redirect for
 													 * app_name and node
 													 * string */
@@ -491,7 +494,7 @@ typedef struct
 									 * possible. If off, SQL comments
 									 * effectively prevent the judgment (pre
 									 * 3.4 behavior). For backward
-									 * compatibilty sake, default is off. */
+									 * compatibility sake, default is off. */
 
 	DLBOW_OPTION disable_load_balance_on_write; /* Load balance behavior when
 												 * write query is issued in an
@@ -524,7 +527,7 @@ typedef struct
 	bool		failover_when_quorum_exists;	/* Do failover only when wd
 												 * cluster holds the quorum */
 	bool		failover_require_consensus; /* Only do failover when majority
-											 * agrees */
+											 * aggrees */
 	bool		allow_multiple_failover_requests_from_node; /* One Pgpool-II node
 															 * can send multiple
 															 * failover requests to
@@ -535,6 +538,18 @@ typedef struct
 											 * votes in a cluster with an even
 											 * number of nodes.
 											 */
+	bool		wd_remove_shutdown_nodes;
+											/* revoke membership of properly shutdown watchdog
+											 * nodes.
+											 */
+	int 		wd_lost_node_removal_timeout;
+											/* timeout in seconds to revoke membership of
+											 * LOST watchdog nodes
+											 */
+	int 		wd_initial_node_showup_time;
+											/* time in seconds to revoke membership of
+											 * NO-SHOW watchdog node
+											 */
 
 	WdLifeCheckMethod wd_lifecheck_method;	/* method of lifecheck.
 											 * 'heartbeat' or 'query' */
@@ -575,12 +590,11 @@ typedef struct
 	int			num_hb_dest_if;				/* number of interface devices */
 	char	  **wd_monitoring_interfaces_list;	/* network interface name list
 												 * to be monitored by watchdog */
-	bool		health_check_test;			/* if on, enable health check testing */
 
 }			POOL_CONFIG;
 
 extern POOL_CONFIG * pool_config;
-extern char config_file_dir[]; /* directory path of config file pgpool.conf */
+extern char *config_file_dir; /* directory path of config file pgpool.conf */
 
 typedef enum
 {
