@@ -63,8 +63,6 @@
 
 /*
  * watchdog state
- * If you add a new state. Remember to add entry in
- * wd_state_names (watchdog.c)
  */
 typedef enum
 {
@@ -122,12 +120,8 @@ typedef enum
 	WD_EVENT_I_AM_APPEARING_FOUND
 }			WD_EVENTS;
 
-/*
- * If you add a new lost reason. Remember to add entry in
- * wd_node_lost_reasons (watchdog.c)
- */
 typedef enum {
-	NODE_LOST_UNKNOWN_REASON = 0,
+	NODE_LOST_UNKNOWN_REASON,
 	NODE_LOST_BY_LIFECHECK,
 	NODE_LOST_BY_SEND_FAILURE,
 	NODE_LOST_BY_MISSING_BEACON,
@@ -136,6 +130,13 @@ typedef enum {
 	NODE_LOST_SHUTDOWN
 } WD_NODE_LOST_REASONS;
 
+typedef enum {
+	WD_NODE_MEMBERSHIP_ACTIVE,
+	WD_NODE_REVOKED_SHUTDOWN,
+	WD_NODE_REVOKED_NO_SHOW,
+	WD_NODE_REVOKED_LOST
+}WD_NODE_MEMBERSHIP_STATUS;
+
 typedef struct SocketConnection
 {
 	int			sock;			/* socket descriptor */
@@ -154,13 +155,18 @@ typedef struct WatchdogNode
 									 * from the node */
 	struct timeval last_sent_time;	/* timestamp when last packet was sent on
 									 * the node */
+	struct timeval lost_time;	/* timestamp when the remote node was lost on coordinator
+								 */
+	WD_NODE_MEMBERSHIP_STATUS	membership_status; /* status of node membership
+											*in watchdog cluster
+											Only valid for remote nodes */
 	bool   has_lost_us;             /*
 									 * True when this remote node thinks
 									 * we are lost
 									 */
 	int    sending_failures_count;  /* number of times we have failed
 									 * to send message to the node.
-									 * Gets reset after successfull sent
+									 * Gets reset after successful sent
 									 */
 	int    missed_beacon_count;     /* number of times the node has
 									 * failed to reply for beacon.
